#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2012  P.L. Lucas
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys
import os
from gi.repository import GLib, GObject, Gio, Pango, GdkPixbuf, Gtk

config_path='/usr/share/anotherterminal'
if 'ANOTHERTERMINAL_PATH' in os.environ:
	config_path=os.environ['ANOTHERTERMINAL_PATH']

sys.path.append(config_path)

from projectxmlhandler import *
from python2and3 import *

def toXml(text):
	t=text.replace('&', '&amp;')
	t=t.replace('<', '&lt;')
	t=t.replace('>', '&gt;')
	return t

class Builder:
	
	def cancel_callback(self, button):
		Gtk.main_quit()
	def delete_callback(self, window, data=None):
		Gtk.main_quit()
	
	def build_window(self, file_name):
		builder = Gtk.Builder()
		if builder.add_from_file(file_name)==0:
			print ("Failed to load UI XML file: {0}".format(file_name))
			sys.exit(1)
		return builder
	
	def open_callback(self, widget):
		dialog = Gtk.FileChooserDialog("Open",
			self.window,
			Gtk.FileChooserAction.OPEN
			,
			[ Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
			Gtk.STOCK_OPEN, Gtk.ResponseType.ACCEPT]
			)
		if self.filename!=None:
			dialog.set_filename(self.filename)
		if dialog.run () == Gtk.ResponseType.ACCEPT:
			self.load_xml_file(dialog.get_filename())
		dialog.destroy ()
	
	def load_xml_file(self, filename):
		self.filename = filename
		parser = make_parser() 
		handler=ProjectXMLHandler()
		parser.setContentHandler(handler)
		parser.parse(open(self.filename))
		
		self.insert_tree(handler.menubar, handler.toolbar)
		self.treeview.expand_all()
		
		self.command=handler.command
		self.tittle=handler.title
	
	def insert_tree(self, menuitems, toolbaritems):
		treeiter = self.treestore.get_iter_first()
		while treeiter!=None:
			self.treestore.remove(treeiter)
			treeiter = self.treestore.get_iter_first()
		treeiter = self.treestore.append(None, ['Menu', None])
		if menuitems!=None:
			self.insert_tree_brach(treeiter, self.treestore, menuitems.menuitems)
		treeiter = self.treestore.append(None, ['Toolbar', None])
		if toolbaritems!=None:
			self.insert_tree_brach(treeiter, self.treestore, toolbaritems.menuitems)
	
	def insert_tree_brach(self, treeiter, store, items):
		for item in items:
			if isinstance(item, MenuItem):
				store.append(treeiter, [item.name, item])
			elif isinstance(item, Menu):
				itemmenu=MenuItem(item.name)
				menuiter=store.append(treeiter, [item.name, itemmenu])
				self.insert_tree_brach(menuiter, store, item.menuitems)
			elif isinstance(item, Separator):
				item=Separator()
				store.append(treeiter, ['------', item])
	
	def save_callback(self, widget):
		dialog = Gtk.FileChooserDialog("Save",
			self.window,
			Gtk.FileChooserAction.SAVE
			,
			[ Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
			Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT]
			)
		if self.filename!=None:
			dialog.set_filename(self.filename)
		if dialog.run () == Gtk.ResponseType.ACCEPT:
			self.filename = dialog.get_filename ()
			fout=open(self.filename, "w")
			fout.write(toUnicodeBytes(self.tree_to_xml(self.treestore)))
			fout.close()
		dialog.destroy ()
	
	
	def tree_to_xml(self, store):
		menuiter = store.get_iter_first()
		xml=toUnicode('<?xml version="1.0" encoding="UTF-8"?>\n<terminalmenu>\n')
		if self.command!=None:
			xml+=toUnicode('<command>{0}</command>\n').format(toXml(self.command))
		if self.tittle!=None:
			xml+=toUnicode('<title>{0}</title>\n').format(toXml(self.tittle))
		childiter = store.iter_children(menuiter)
		xml+=toUnicode('<menubar>\n')
		xml+=self.tree_to_xml_row(store, childiter, "\t")
		xml+=toUnicode('</menubar>\n')
		toolbariter = store.iter_next(menuiter)
		childiter = store.iter_children(toolbariter)
		if childiter!=None:
			xml+=toUnicode('<toolbar>\n')
			xml+=self.tree_to_xml_row(store, childiter, "\t")
			xml+=toUnicode('</toolbar>\n')
		xml+=toUnicode("</terminalmenu>")
		return xml
	
	def tree_to_xml_row(self, store, treeiter, indent):
		xml=toUnicode('')
		while treeiter != None:
			item=store[treeiter][1]
			if item!=None and isinstance(item, MenuItem):
				if store.iter_has_child(treeiter):
					xml+=toUnicode("{indent}<menu name='{name}'>\n").format(indent=toXml(indent), name=toXml(item.name))
					childiter = store.iter_children(treeiter)
					xml+=self.tree_to_xml_row(store, childiter, indent + "\t")
					xml+=toUnicode("{indent}</menu>\n").format(indent=toXml(indent))
				else:
					xml+=toUnicode("{indent}<menuitem name='{name}' ").format(indent=toXml(indent), name=toXml(item.name))
					if item.stockicon!=None:
						xml+=toUnicode("stockicon='{stockicon}' ").format(stockicon=toXml(item.stockicon))
					if item.file_dialog_mode!=None and item.file_dialog_mode!='':
						xml+=toUnicode("file_dialog_mode='{file_dialog_mode}' ").format(file_dialog_mode=toXml(item.file_dialog_mode))
					if item.file_dialog_replace_str!=None and item.file_dialog_replace_str!='':
						xml+=toUnicode("file_dialog_replace_str='{file_dialog_replace_str}' ").format(file_dialog_replace_str=toXml(item.file_dialog_replace_str))
					xml+=toUnicode('>\n')
					tag=toUnicode('command')
					if item.exec_ok:
						tag=toUnicode('exec')
					if item.command!=None:
						xml+=toUnicode("{indent}<{tag}>{command}</{tag}>\n").format(indent=toXml(indent+'\t'), command=toXml(item.command), tag=toXml(tag))
					if item.tooltip!=None and item.tooltip!='':
						xml+=toUnicode("{indent}<tooltip>{tooltip}</tooltip>\n").format(indent=toXml(indent+'\t'), tooltip=toXml(item.tooltip))
					xml+=toUnicode("{indent}</menuitem>\n").format(indent=toXml(indent))
			elif item!=None and isinstance(item, Separator):
				xml+=toUnicode("{indent}<separator />\n").format(indent=toXml(indent))
			treeiter = store.iter_next(treeiter)
		return xml
	
	def row_activated_callback(self, path, column, data):
		selection = self.treeview.get_selection()
		model, treeiter = selection.get_selected()
		if treeiter != None and model[treeiter][1]!=None:
			#Open menuitem dialog
			item=model[treeiter][1]
			if item!=None and isinstance(item, Separator):
				return False
			builder = self.build_window(config_path+"/menuitemdialog.xml")
			dialog = builder.get_object("dialog")
			stockicons_liststore = builder.get_object('stockicons_liststore')
			stockicon_combobox = builder.get_object("stockicon_combobox")
			n=0
			for icon in Gtk.stock_list_ids():
				image = Gtk.Image()
				pixbuf=image.render_icon_pixbuf(icon, Gtk.IconSize.BUTTON)
				if pixbuf!=None:
					stockicons_liststore.append([pixbuf])
					if icon==item.stockicon:
						stockicon_combobox.set_active(n)
					n+=1
			name_entry = builder.get_object("name_entry")
			name_entry.set_text(item.name)
			tooltip_entry = builder.get_object("tooltip_entry")
			if item.tooltip!=None:
				tooltip_entry.set_text(item.tooltip)
			command_textview = builder.get_object("command_textview")
			command_buffer = command_textview.get_buffer()
			if item.command!=None:
				command_buffer.set_text(item.command)
			exec_checkbutton = builder.get_object("exec_checkbutton")
			exec_checkbutton.set_active(item.exec_ok)
			open_dialog_mode_combobox = builder.get_object("open_dialog_mode_combobox")
			if item.file_dialog_mode=='open':
				open_dialog_mode_combobox.set_active(1)
			elif item.file_dialog_mode=='save':
				open_dialog_mode_combobox.set_active(2)
			else:
				open_dialog_mode_combobox.set_active(0)
			replace_str_entry = builder.get_object("replace_str_entry")
			if item.file_dialog_replace_str!=None:
				replace_str_entry.set_text(item.file_dialog_replace_str)
			ok=dialog.run()
			if ok==-5:
				item.name=name_entry.get_text()
				model[treeiter][0]=item.name
				item.tooltip=tooltip_entry.get_text()
				item.command=command_buffer.get_text(command_buffer.get_iter_at_offset(0), command_buffer.get_iter_at_offset(-1), False)
				item.exec_ok=exec_checkbutton.get_active()
				if open_dialog_mode_combobox.get_active()==1:
					item.file_dialog_mode='open'
				elif open_dialog_mode_combobox.get_active()==2:
					item.file_dialog_mode='save'
				else:
					item.file_dialog_mode=None
				item.file_dialog_replace_str=replace_str_entry.get_text()
				n=stockicon_combobox.get_active()
				if n>=0:
					item.stockicon=Gtk.stock_list_ids()[n]
				else:
					item.stockicon=None
			dialog.destroy()
		return False
	
	def settings_callback(self, widget):
		builder = self.build_window(config_path+"/settingsdialog.xml")
		dialog = builder.get_object("dialog")
		tittle_entry = builder.get_object("tittle_entry")
		if self.tittle!=None:
			tittle_entry.set_text(self.tittle)
		command_entry = builder.get_object("command_entry")
		if self.command!=None:
			command_entry.set_text(self.command)
		ok=dialog.run()
		if ok==-5:
			if command_entry.get_text()=='':
				self.command=None
			else:
				self.command=command_entry.get_text()
			if tittle_entry.get_text()=='':
				self.tittle=None
			else:
				self.tittle=tittle_entry.get_text()
		dialog.destroy()
		return False
	
	def remove_callback(self, widget):
		selection = self.treeview.get_selection()
		model, treeiter = selection.get_selected()
		if treeiter != None and self.treestore[treeiter][1]!=None:
			self.treestore.remove(treeiter)
	
	def add_callback(self, widget):
		selection = self.treeview.get_selection()
		model, treeiter = selection.get_selected()
		if treeiter != None:
			item=MenuItem('Item')
			self.treestore.append(treeiter, ['Item', item])
			self.treeview.expand_all()
	
	def add_separator_callback(self, widget):
		selection = self.treeview.get_selection()
		model, treeiter = selection.get_selected()
		if treeiter != None:
			item=Separator()
			self.treestore.append(treeiter, ['------', item])
			self.treeview.expand_all()
	
	def select_callback(self, selection):
		model, treeiter = selection.get_selected()
		remove_toolbutton = self.builder.get_object("remove_toolbutton")
		add_toolbutton = self.builder.get_object("add_toolbutton")
		if treeiter != None:
			item=model[treeiter][1]
			if item!=None and isinstance(item, Separator):
				add_toolbutton.set_state(Gtk.StateType.INSENSITIVE)
			else:
				add_toolbutton.set_state(Gtk.StateType.NORMAL)
			if item!=None:
				remove_toolbutton.set_state(Gtk.StateType.NORMAL)
			else:
				remove_toolbutton.set_state(Gtk.StateType.INSENSITIVE)
		else:
			add_toolbutton.set_state(Gtk.StateType.INSENSITIVE)
			remove_toolbutton.set_state(Gtk.StateType.INSENSITIVE)
	
	
	def init_treeview(self):
		scrolledwindow=self.builder.get_object('treeview_scrolledwindow')
		self.treestore = Gtk.TreeStore(str, object)
		self.treeview=Gtk.TreeView(self.treestore)
		self.treeview.set_show_expanders(True)
		self.treeview.set_level_indentation(10)
		self.treeview.set_reorderable(True)
		self.treeview.show()
		scrolledwindow.add(self.treeview)
		
		renderer = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn("Menu", renderer, text=0)
		column.set_expand(True)
		self.treeview.append_column(column)
		
		self.treestore.append(None, ['Menu', None])
		self.treestore.append(None, ['Toolbar', None])
		
		select = self.treeview.get_selection()
		select.connect("changed", self.select_callback)
		self.treeview.connect("row-activated", self.row_activated_callback)
		
		remove_toolbutton = self.builder.get_object("remove_toolbutton")
		remove_toolbutton.set_state(Gtk.StateType.INSENSITIVE)
		
		add_toolbutton = self.builder.get_object("add_toolbutton")
		add_toolbutton.set_state(Gtk.StateType.INSENSITIVE)
		
		#treeiter = self.comandos_store.append(["Crear usuario", "hola"])
		#for row in self.comandos_store:
		#	# Print values of all columns
		#	print (row[0][0])

	def __init__(self, filename=None):
		self.tittle=None
		self.command=None
		# use GtkBuilder to build our interface from the XML file 
		self.builder = Gtk.Builder()
		if self.builder.add_from_file(config_path+"/builder.xml")==0:
			print ("Failed to load UI XML file: builder.xml")
			sys.exit(1)
		
		self.filename=None
		
		# connect signals
		self.builder.connect_signals(self)

		
		self.window = self.builder.get_object("window")
		
		
		#self.comandos_treeview = self.builder.get_object("comandos_treeview")
		#self.conexiones_treeview = self.builder.get_object("conexiones_treeview")
		
		self.init_treeview()
		
		self.window.show()
		
		if filename!=None:
			self.load_xml_file(filename)

if __name__ == "__main__":
	filename=None
	for argv in sys.argv[1:]:
		if argv=='--help':
			print("""
{0} [--help] [filename]
This tool helps to user build a new menu to be included in AnotherTerminal.
Options:
--help	Show this help
""".format(sys.argv[0]))
			exit(0)
		else:
			filename=argv
	editor = Builder(filename)
	Gtk.main()
