#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2012  P.L. Lucas
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from xml.sax import make_parser
from xml.sax.handler import ContentHandler

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, GObject, Gio, Pango, GdkPixbuf, Gtk, Gdk

# Base class of Menubar. Only contains a list of menus
class Menubar:
	def __init__(self):
		#List of menus
		self.menuitems=[]
		self.id=-1 #A simple id to store gtkaction name

class Toolbar:
	def __init__(self):
		self.menuitems=[]
		self.id=-1

class Menu:
	def __init__(self, name):
		self.name=name
		self.menuitems=[]
		self.id=-1

class MenuItem:
	def __init__(self, name):
		self.name=name		#Name of menu item
		self.command=None	#Text to feed terminal or eval as python
		self.exec_ok=False	#If true, command is evaluated by python eval
		self.stockicon=None	#Stockicon name
		self.tooltip=None	#Text of tooltip
		self.file_dialog_mode=None	#If not None, this is a file dialog
		self.file_dialog_replace_str=None	#This string will be replaced in command
		self.id=-1

class Separator:
	def __init__(self):
		self.id=-1

class ProjectXMLHandler(ContentHandler):
	def __init__(self):
		self.menubar=None
		self.toolbar=None
		self.pila=[]
		self.text=''
		self.command='bash'
		self.title=None
		self.id=0
	
	def startElement(self, name, attrs):
		item=None
		if name=='menuitem':
			item=MenuItem(attrs['name'])
			if 'stockicon' in attrs:
				#item.stockicon=eval('Gtk.'+attrs['stockicon'])
				item.stockicon=attrs['stockicon']
			if 'file_dialog_mode' in attrs:
				item.file_dialog_mode=attrs['file_dialog_mode'].strip()
				#if 'open'==attrs['file_dialog_mode'].strip():
				#	item.file_dialog_mode=Gtk.FileChooserAction.OPEN
				#else:
				#	item.file_dialog_mode=Gtk.FileChooserAction.SAVE
			if 'file_dialog_replace_str' in attrs:
				item.file_dialog_replace_str=attrs['file_dialog_replace_str']
			item.id=self.id
			#print(attrs['name'])
		elif name=='menu':
			item=Menu(attrs['name'])
			item.id=self.id
		elif name=='separator':
			item=Separator()
		elif name=='menubar':
			item=Menubar()
			item.id=self.id
			self.menubar=item
		elif name=='toolbar':
			item=Toolbar()
			item.id=self.id
			self.toolbar=item
		
		if item!=None:
			#print(name)
			self.pila.append(item)
			self.id+=1
		else:
			self.text=''
	
	def endElement(self, name):
		item=None
		if len(self.pila)>0:
			item=self.pila.pop()
		
		if item==None:
			if name=='command':
				self.command=self.text
			if name=='title':
				self.title=self.text
		
		if isinstance(item, Menubar):
			if name!='menubar':
				self.pila.append(item)
		elif isinstance(item, Toolbar):
			if name!='toolbar':
				self.pila.append(item)
		elif isinstance(item, MenuItem):
			if name=='command':
				item.command=self.text
			elif name=='exec':
				item.command=self.text
				item.exec_ok=True
			elif name=='tooltip':
				item.tooltip=self.text
			if name!='menuitem':
				self.pila.append(item)
		last=None
		n=len(self.pila)-1
		if n>=0:
			last=self.pila[n]
		if isinstance(last, Menu) or isinstance(last, Menubar) or isinstance(last, Toolbar):
			last.menuitems.append(item)
			#print('Añadido item')
		self.text=''
	
	def characters (self, ch):
		self.text += ch
